@@ -103,11 +103,21 @@ class SonyEndPoint:
         self.id = self.id % self.ID_MAX + 1
         return ret
 
+    def unimplemented_method(self, *_args, **_kwargs):
+        """Called when finding an unimplemented method."""
+        return {"error": [501, "Not Implemented"], "id": self.id}
+
     def __getattr__(self, name):
         """Call when accessing a non-existing endpoint method."""
-        def unimplemented_method(*args, **kwargs):
-            return {"error": [501, "Not Implemented"], "id": self.id}
-        return unimplemented_method
+        return self.unimplemented_method
+
+    def __getstate__(self):
+        """When pickling this class, use this state."""
+        return self.__dict__.copy()
+
+    def __setstate__(self, d):
+        """When un-pickling this class, use this method to add data."""
+        self.__dict__.update(d)
 
 
 class SonyImagingDevice:
@@ -380,7 +390,7 @@ class SonyImagingDevice:
         req.add_header('Content-Type', 'application/json; charset=utf-8')
         body_utf8 = json.dumps(args)
         body = body_utf8.encode("utf-8")
-        logging.info(f"MSG {body_utf8}")
+        logging.debug(f"MSG {body_utf8}")
         res = {}
         try:
             timeout = self.timeout_seconds
@@ -399,7 +409,7 @@ class SonyImagingDevice:
             res = {"error": [504, "Invalid data in returned JSON"]}
         except urllib.error.URLError as err:
             res = {"error": [601, err.reason], "id": id}
-        logging.info(f"RES {res}")
+        logging.debug(f"RES {res}")
         return res
 
 
@@ -416,7 +426,7 @@ def get_status(dev):
     """
     ev = dev.camera.getEvent(params=[False])
     if "result" not in ev:
-        raise SonyDeviceError("Unexpected response from getEvent")
+        raise SonyDeviceError("Unexpected response from getEvent: %s", ev)
     return ev["result"][1]["cameraStatus"]
 
 
